//*********************** 2612vi.c ****************************
//* This file contains Lab View DLL to communicate with
//* Sensoray model 2612 relay board.
//* Written by Alexander Kostromitin
//* Copyright (C) Sensoray Company 2006
//*************************************************************


#define DEBUG     1     // generate some additional info


#include <time.h>
#include <string.h>
#include <stdio.h>
#include <windows.h>
#include <commdlg.h>    // includes common dialog functionality
#include <commctrl.h>
#include <dlgs.h>       // includes common dialog template defines
#include <cderr.h>      // includes the common dialog error codes
#include <io.h>
#include <math.h>


//#include "Win2600.h"
//#include "App2600.h"
#include "f2600.h"


#define NUM_ADC     4           // number of input channels
#define STD_TIMEOUT 100         // default value for S26_SchedExecute

int check_cnl(int cnl);
int check_connect(void);
static int dllinit(void);
static void dllclose(void);
static int findmodule(int mm, int id, int from);
static void clearerrors(int mm, int iom);
static int io_exec(void *x);
static void error(char *fmt, ... );
static void logmsg(char *fmt, ... );

int errstop;

#define LOG { if(errstop) error("%d %d", __LINE__, errstop); }
#define STOP { errstop = __LINE__; }

#define chkerr(f)   { int err=(f); if(err) error("Error #%x", err); }


// Common parameters
static char errstr[256];                // error message string
static int connectok;                   // connected to a board
static int errorflag;


// DLL entry point
BOOL WINAPI DllEntryPoint(HANDLE hDLL, DWORD dwReason, LPVOID lpReserved)
{
  if(dwReason == DLL_PROCESS_ATTACH)
    return dllinit() == 0;
  if(dwReason == DLL_PROCESS_DETACH)
    dllclose();
  return 1;
}

// alternative entry point
BOOL WINAPI DllMain(HANDLE hDLL, DWORD dwReason, LPVOID lpReserved)
{
  if(dwReason == DLL_PROCESS_ATTACH)
    return dllinit() == 0;
  if(dwReason == DLL_PROCESS_DETACH)
    dllclose();
  return 1;
}



/////////////////////////////////////////////////////////////////
// exported functions
//
// note: for compatibility reasons all exported functions
// declared as __stdcall (WINAPI), do not use regular C
// convention to call them from VI.
/////////////////////////////////////////////////////////////////


// Open board
//  ClientDotAdrs: IP address of host computer's connection (use "0" for default)
//  MMDotAdrs: IP address of Main Module (2601)
//  mm: index of registering Main Module in internal DLL's table
//  iom: module (2601 IOM connector) number (use -1 to open first module found)
//  wdmsec: watcdog interval in milliseconds (maximum time between commands to
//   avoid internal reset; use 0 to disable)
__declspec (dllexport) int __stdcall openboard(char *ClientDotAdrs, char *MMDotAdrs, int mm, int iom, int wdmsec)
{
  int err;
  void *x;
  int port;

  connectok = 0;

  // establish connection with 2601 base module
  S26_BoardOpen(mm, ClientDotAdrs, MMDotAdrs);
  err = !S26_ResetNetwork(mm);
  if(err)
    {
    error("Can't initialize the MM");
    return(-1);
    }
  // set watchdog
  if(wdmsec != 0)
    {
    wdmsec /= 100;
    if(wdmsec <= 0)
      wdmsec = 1;
    else if(wdmsec > 255)
      wdmsec = 255;
    }
  x = S26_SchedOpen(mm, 1);
  S26_Sched2601_SetWatchdog(x, wdmsec);
  io_exec(x);

  // find 1st board
  port = findmodule(mm, 2612, iom);
  if(port < 0)
    {
    error("No board found");
    return(-1);
    }
  connectok = 1;
  return(port);
}


// Get last error message
// Clear error status if required
__declspec (dllexport) char *__stdcall getlasterror(int mm, int iom, int clear)
{
  static char err[256];

  strcpy(err, errstr);
  if(clear)
    strcpy(errstr, "Ok\n");
  else
    *errstr = 0;
  if(errorflag)
    {
    clearerrors(mm, iom);
    errorflag = 0;
    }
  return err;
}


//////////////////////////////////////////////////////////////////////
// Board specific functions.
// See Model 2600 Family Programming Guide for details.
//////////////////////////////////////////////////////////////////////

static unsigned long smpnum[4][16][NUM_ADC];

// Get internally buffered value for an ADC. If ADC is not ready, wait and try again
// several times.
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//  cnl: channel
__declspec (dllexport) double __stdcall getdata(int mm, int iom, int cnl)
{
  int err;
  u32 sn;
  double res;
  int attcnt;
  void *x;

  
  if(check_cnl(cnl) || check_connect())
    return 0;

  for(attcnt = 100; attcnt > 0; attcnt--)
    {
    res = S26_2612_GetCalibratedValue(mm, iom, cnl, &sn);
    if(sn != smpnum[mm][iom][cnl])
      break;
    Sleep(1);
    // Try to refresh
    x = S26_SchedOpen(mm, 1);
    if(x == 0)
      {
      error("x = 0, mm = %d", mm); 
      return 0;
      }
    err = S26_Sched2612_RefreshData(x, iom);
    if(err)
      {
      error("S26_Sched2612_RefreshData: %x %x %d", err, x, iom); 
      return 0;
      }
    err = io_exec(x);
    if(err)
      {
      error("io_exec: %x %x", err, x); 
      return 0;
      }
    }
  if(attcnt == 0)
    {
    error("ADC %d not ready...", cnl);
    return 0;
    }
  smpnum[mm][iom][cnl] = sn;
  
  return res;
}


// Get internally buffered number of sample for an ADC
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//  cnl: channel
__declspec (dllexport) unsigned long __stdcall getsmpnum(int mm, int iom, int cnl)
{
  return smpnum[mm][iom][cnl];
}


// Refresh internally buffered values for all ADCs
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
__declspec (dllexport) void __stdcall refreshdata(int mm, int iom)
{
  void *x;                // Transaction object.

  if(check_connect())
    return;
  x = S26_SchedOpen(mm, 1);
  S26_Sched2612_RefreshData(x, iom);
  io_exec(x);
}


// Set zero (offset) value for one ADC
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//  cnl: channel
// nsmp: number of samples for averaging
__declspec (dllexport) void __stdcall setgagezero(int mm, int iom, int cnl, int nsmp)
{
  if(check_cnl(cnl) || check_connect())
    return;
  S26_2612_RegisterZero(mm, iom, STD_TIMEOUT, cnl, nsmp);
}


// Calculate and set scale value for one ADC
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//  cnl: channel
// nsmp: number of samples for averaging
// load: actual load, applied to the gauge
__declspec (dllexport) void __stdcall setgagespan(int mm, int iom, int cnl, int nsmp, double load)
{
  if(check_cnl(cnl) || check_connect())
    return;
  S26_2612_RegisterSpan(mm, iom, STD_TIMEOUT, cnl, nsmp, load);

}


// Set tare (calibrated offset) value for one ADC
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//  cnl: channel
// nsmp: number of samples for averaging
__declspec (dllexport) void __stdcall setgagetare(int mm, int iom, int cnl, int nsmp)
{
  if(check_cnl(cnl) || check_connect())
    return;
  S26_2612_RegisterTare(mm, iom, STD_TIMEOUT, cnl, nsmp);
}



// Input parameters

#define NMODES  10    // number of modes

unsigned int adcmodes[NMODES] =
    {
    // Over Sample Ratio (OSR) / Conversion Rate in 1X mode:
    OSR_64,  			// 64        3.52kHz
    OSR_128,            // 128       1.76kHz
    OSR_256,            // 256       880Hz
    OSR_512,            // 512       440Hz
    OSR_1024,           // 1024      220Hz
    OSR_2048,           // 2048      110Hz
    OSR_4096,           // 4096      55Hz
    OSR_8192,           // 8192      27.5Hz
    OSR_16384,          // 16384     13.75Hz
    OSR_32768           // 32768     6.875Hz
    };


// Set channel mode
//  mm: index of the Main Module used in <openboard>
//  iom: module number
//  cnl: channel
//  mode: mode
//  x2: disable/enable speed x2
__declspec (dllexport) void __stdcall setinmode(int mm, int iom, int cnl, int mode, int x2)
{
  void *x;                // Transaction object.

  if(check_cnl(cnl) || check_connect() || mode < 0 || mode >= NMODES)
    return;
  x = S26_SchedOpen(mm, 1);
  S26_Sched2612_SetMode(x, iom, cnl, adcmodes[mode] | (x2 ? MODE_2X : 0));
  io_exec(x);
}


// Set all voltage controls as combination of 2-bit fields
//  mm: index of the Main Module used in <openboard>
//  iom: module number
__declspec (dllexport) void __stdcall setvoltages(int mm, int iom, int voltages)
{
  void *x;                // Transaction object.

  if(check_connect())
    return;
  x = S26_SchedOpen(mm, 1);
  S26_Sched2612_SetVoltages(x, iom, voltages);
  io_exec(x);                                            
}


// Save calibrations for one channel
//  mm: index of the Main Module used in <openboard>
//  iom: module number
//  cnl: channel
__declspec (dllexport) void __stdcall savecalibrations(int mm, int iom, int cnl)
{
  int err;
  
  if(check_connect())
    return;
  err = S26_2612_SaveCalibrations(mm, iom, STD_TIMEOUT, cnl);
  if(err)
    error("Error %x in savecalibrations()", err);
  Sleep(20);
}


// Restore calibrations for all channels
//  mm: index of the Main Module used in <openboard>
//  iom: module number
__declspec (dllexport) void __stdcall restorecalibrations(int mm, int iom)
{
  int cnl, err;
  
  if(check_connect())
    return;
  for(cnl = 0; cnl < NUM_ADC; cnl++)
    {
    err = S26_2612_RestoreCalibrations(mm, iom, STD_TIMEOUT, cnl);
    if(err)
      error("Not calibrated");
    }
}



/////////////////////////////////////////////////////////////////
// internal functions
/////////////////////////////////////////////////////////////////

// Check channel number
int check_cnl(int cnl)
{
  if(cnl < 0 || cnl >= NUM_ADC)
    {
    error("Incorrect Channel Number %d", cnl);
    return 1;
    }
  return 0;
}

// Check connection status
int check_connect(void)
{
  if(!connectok)
    {
    error("No board connected...");
    return 1;
    }
  return 0;
}


// Open and init main DLL
static int dllinit(void)
{
  int err;

  // open DLL
  #if DEBUG
  // local directory
  err = S26_DLLOpen("c:\\2600\\labview", 1);
  //err = S26_DLLOpen("d:\\ntprog\\windrv\\2600\\last", 1);
  #else
  // some common directory (windows\system32)
  err = S26_DLLOpen(NULL, 1);
  #endif

  if(err)
    {
    error("2600 DLL open error #%d", err);
    return -1;
    }

  return 0;
}                                                                           


// Close main DLL
static void dllclose(void)
{
  S26_DriverClose();
}


// Find a module
//  id: module ID
//  from: number of module to start from
static int findmodule(int mm, int id, int from)
{
  int err;
  int iom;
  unsigned short num;
  unsigned short ioms[16];
  unsigned char sts[16];

  if(from < 0)
    {
    err = S26_RegisterAllIoms(mm, 1000, &num, ioms, sts, 1);
    if(err)
      return -1;
    from = 0;
    }
  // Count the connected IOMs, stop on first board found.
  for(iom = from; iom < 16; iom++)
    {
    if(ioms[iom] == id)
      {
      return iom;
      }
    }
  return -1;
}


// Clear error status
//  mm: index of the Main Module used in <openboard>
//  iom: module number
void clearerrors(int mm, int iom)
{
  void *x;                // Transaction object.

  if(check_connect())
    return;
  x = S26_SchedOpen(mm, 1);
  S26_Sched2600_ClearStatus(x, iom, ~0);
  io_exec(x);
}


// Error message
static void error(char *fmt, ... )
{
  va_list argptr;

  va_start(argptr, fmt);
  vsprintf(errstr, fmt, argptr);
  va_end(argptr);
  strcat(errstr, "\n");
  errorflag = 1;
  // print error messages to file
  #if DEBUG
    {
    FILE *f;

    if((f = fopen("c:\\2612vidll.err", "a")) == NULL)
      return;
    fprintf(f, "%s", errstr);
    fclose(f);
    }
  #endif

}


// Log message
static void logmsg(char *fmt, ... )
{
  va_list argptr;
  char str[256];
  FILE *f;

  va_start(argptr, fmt);
  vsprintf(str, fmt, argptr);
  va_end(argptr);

  if((f = fopen("c:\\2612vidll.log", "a")) == NULL)
    return;
  fprintf(f, "%s\n", str);
  fclose(f);
}


// Execute all scheduled i/o.
// Returns zero if successful.
static int io_exec(void *x)
{
  char status[16];		// Iom status info.
  GWERR err;

  // Execute the scheduled i/o.  Report error if one was detected.
  err = S26_SchedExecute(x, STD_TIMEOUT, status );
  if(err)
    error("Connection error 0x%08x", err);
  return err;
}





