//*********************** 2620vi.c ****************************
//* This file contains Lab View DLL to communicate with
//* Sensoray model 2620 counter board.
//* Written by Alexander Kostromitin
//* Copyright (C) Sensoray Company 2006
//*************************************************************


#define DEBUG     1     // generate some additional info


#include <time.h>
#include <string.h>
#include <stdio.h>
#include <windows.h>
#include <commdlg.h>    // includes common dialog functionality
#include <commctrl.h>
#include <dlgs.h>       // includes common dialog template defines
#include <cderr.h>      // includes the common dialog error codes
#include <io.h>
#include <math.h>


//#include "Win2600.h"
#include "f2600.h"


static int dllinit(void);
static void dllclose(void);
static int findmodule(int mm, int id, int from);
static int io_exec(void *x);
static void error(char *fmt, ... );



#define chkerr(f)   { int err=(f); if(err) error("Error #%x", err); }


// Common parameters
static char errstr[256];                // error message string

 

// DLL entry point
BOOL WINAPI DllEntryPoint(HANDLE hDLL, DWORD dwReason, LPVOID lpReserved)
{
  if(dwReason == DLL_PROCESS_ATTACH)
    return dllinit() == 0;
  if(dwReason == DLL_PROCESS_DETACH)
    dllclose();
  return 1;
}

// alternative entry point
BOOL WINAPI DllMain(HANDLE hDLL, DWORD dwReason, LPVOID lpReserved)
{
  if(dwReason == DLL_PROCESS_ATTACH)   
    return dllinit() == 0;
  if(dwReason == DLL_PROCESS_DETACH)
    dllclose();
  return 1;
}



/////////////////////////////////////////////////////////////////
// exported functions
//
// note: for compatibility reasons all exported functions
// declared as __stdcall (WINAPI), do not use regular C
// convention to call them from VI.
/////////////////////////////////////////////////////////////////


// Open board
//  ClientDotAdrs: IP address of host computer's connection (use "0" for default)
//  MMDotAdrs: IP address of Main Module (2601)
//  mm: index of registering Main Module in internal DLL's table
//  iom: module (2601 IOM connector) number (use -1 to open first module found)
//  wdmsec: watcdog interval in milliseconds (maximum time between commands to
//   avoid internal reset; use 0 to disable)
__declspec (dllexport) int __stdcall openboard(char *ClientDotAdrs, char *MMDotAdrs, int mm, int iom, int wdmsec)
{
  int err;
  void *x;
  int port;

  // establish connection with 2601 base module
  S26_BoardOpen(mm, ClientDotAdrs, MMDotAdrs);
  err = !S26_ResetNetwork(mm);
  if(err)
    {
    error("Can't initialize the MM");
    return(-1);
    }
  // set watchdog
  if(wdmsec != 0)
    {
    wdmsec /= 100;
    if(wdmsec <= 0)
      wdmsec = 1;
    else if(wdmsec > 255)
      wdmsec = 255;
    }
  x = S26_SchedOpen(mm, 1);
  S26_Sched2601_SetWatchdog(x, wdmsec);
  io_exec(x);

  // find 1st board
  port = findmodule(mm, 2620, iom);
  if(port < 0)
    {
    error("No board found");
    return(-1);
    }
  return(port);
}


// Get last error message
// Clear error status if required
__declspec (dllexport) char *__stdcall getlasterror(int clear)
{
  static char err[256];

  strcpy(err, errstr);
  if(clear)
    strcpy(errstr, "Ok");
  return err;
}



//////////////////////////////////////////////////////////////////////
// Board specific functions.
// See Model 2600 Family Programming Guide for details.
//////////////////////////////////////////////////////////////////////


// Return the common control register setting (from middleware's image)
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//	return: internal gate period (ms)
__declspec (dllexport) int __stdcall GetCommonControl(int mm, int iom)
{
  void *x;                // Transaction object.
  unsigned char tstamp;
  unsigned short period;

  x = S26_SchedOpen(mm, 1);
  chkerr(S26_Sched2620_GetCommonControl(x, iom, &period, &tstamp));
  io_exec(x);
  return period;
}


#define CTC_TRIG_PRELOAD        0x01    // force counter load
#define CTC_TRIG_LATCH          0x02    // force counter latch


// Return counts from the specified counter channel
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//	chan: counter channel number in range 0...3
//  latch: use 0 to read counter's value, latched by hardware (in some specific moment) or
//    1 to latch and read the current value.
//	return: latched value from counter
__declspec (dllexport) unsigned long __stdcall GetCounts(int mm, int iom, int chan, int latch)
{
  void *x;                // Transaction object.
  unsigned long v;

  x = S26_SchedOpen(mm, 1);

  if(latch)               // force latch
    S26_Sched2620_SetControlReg(x, iom, chan, CTC_TRIG_LATCH);

  chkerr(S26_Sched2620_GetCounts(x, iom, chan, &v, NULL));
  io_exec(x);
  return v;
}


// Return the operating mode for one counter channel (from middleware's image)
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//	chan: counter channel number in range 0...3
//	return: value of mode register
__declspec (dllexport) int __stdcall GetMode(int mm, int iom, int chan)
{
  void *x;                // Transaction object.
  unsigned short mode;

  x = S26_SchedOpen(mm, 1);
  chkerr(S26_Sched2620_GetMode(x, iom, chan, &mode));
  io_exec(x);
  return mode;
}


// Return middleware's image of preload counter
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//	chan: counter channel number in range 0...3
//	reg: preload register: 0 or 1
//	return: preload value
__declspec (dllexport) unsigned long __stdcall GetPreload(int mm, int iom, int chan, int reg)
{
  void *x;                // Transaction object.
  unsigned long value;

  x = S26_SchedOpen(mm, 1);
  chkerr(S26_Sched2620_GetPreload(x, iom, chan, reg, &value));
  io_exec(x);
  return value;
}


// Return status info for one counter channel
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//	chan: counter channel number in range 0...3
//	return: value of status register
__declspec (dllexport) int __stdcall GetStatus(int mm, int iom, int chan)
{
  void *x;                // Transaction object.
  unsigned short status;

  x = S26_SchedOpen(mm, 1);
  chkerr(S26_Sched2620_GetStatus(x, iom, chan, &status));
  io_exec(x);
  return status;
}


// Program one of the specified counter channel's preload registers
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//	chan: counter channel number in range 0...3
//	reg: preload register number: 0 or 1
//	value: 32-bit value to be preloaded
__declspec (dllexport) void __stdcall SetPreload(int mm, int iom, int chan, int reg, int value)
{
  void *x;                // Transaction object.

  x = S26_SchedOpen(mm, 1);
  chkerr(S26_Sched2620_SetPreload(x, iom, chan, reg, value));
  io_exec(x);
}


// Program the common control register (gate generator period and timestamp resolution)
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//  gperiod: gate generator period even value in ms from 2 to 32766
//  tstamp: timestamp resolution (0, 1, 2, or 3)
__declspec (dllexport) void __stdcall SetCommonControl(int mm, int iom, int gperiod, int tstamp)
{
  void *x;                // Transaction object.

  x = S26_SchedOpen(mm, 1);
  chkerr(S26_Sched2620_SetCommonControl(x, iom, gperiod, tstamp));
  io_exec(x);
}


// Program the control register for one counter channel; this triggers a Latch or Preload operation
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//	chan: counter channel number in range 0...3
//  command: CTC_TRIG_PRELOAD or/and CTC_TRIG_LATCH to force counter preload and latch
__declspec (dllexport) void __stdcall SetControlReg(int mm, int iom, int chan, int command)
{
  void *x;                // Transaction object.

  x = S26_SchedOpen(mm, 1);
  chkerr(S26_Sched2620_SetControlReg(x, iom, chan, command));
  io_exec(x);
}


// Set the operating mode for one counter channel
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//	chan: counter channel number in range 0...3
//  mode: any arbitrary operating mode for a counter channel
__declspec (dllexport) void __stdcall SetMode(int mm, int iom, int chan, int mode)
{
  void *x;                // Transaction object.

  x = S26_SchedOpen(mm, 1);
  chkerr(S26_Sched2620_SetMode(x, iom, chan, mode));
  io_exec(x);
}



////////////////////////////////////////////////////////////////////////////
// The following functions provide user frendly interface to the operating
// mode register instead the common purpose SetMode function.
///////////////////////////////////////////////////////////////////////////

// Set counter mode for encoder interface
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//	chan: counter channel number in range 0...3
//   ActLowX, HardPreload, ClkMode: encoder-dependent parameters (see manual for details)
__declspec (dllexport) void __stdcall SetModeEncoder(int mm, int iom, int chan, int ActLowX, int HardPreload, int ClkMode)
{
  void *x;                // Transaction object.

  x = S26_SchedOpen(mm, 1);
  chkerr(S26_Sched2620_SetModeEncoder(x, iom, chan, ActLowX, HardPreload, ClkMode));
  io_exec(x);
}


// Set counter mode for frequency measurement
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//	chan: counter channel number in range 0...3
//  InternalGate: Time gate signal source: 0 = external signal on index input, 1 = internal gate generator.
__declspec (dllexport) void __stdcall SetModeFreqMeas(int mm, int iom, int chan, int InternalGate)
{
  void *x;                // Transaction object.

  x = S26_SchedOpen(mm, 1);
  chkerr(S26_Sched2620_SetModeFreqMeas(x, iom, chan, InternalGate));
  io_exec(x);
}


// Set counter mode for period measurement;  input signal is applied to index input
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//	chan: counter channel number in range 0...3
//  ActLowX: index pin polarity: 1 = active low, 0 = active high
__declspec (dllexport) void __stdcall SetModePeriodMeas(int mm, int iom, int chan, int ActLowX)
{
  void *x;                // Transaction object.

  x = S26_SchedOpen(mm, 1);
  chkerr(S26_Sched2620_SetModePeriodMeas(x, iom, chan, ActLowX));
  io_exec(x);
}


// Set counter mode for pulse width measurement
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//	chan: counter channel number in range 0...3
//  ActLowX: index pin polarity: 1 = active low, 0 = active high
__declspec (dllexport) void __stdcall SetModePulseMeas(int mm, int iom, int chan, int ActLowX)
{
  void *x;                // Transaction object.

  x = S26_SchedOpen(mm, 1);
  chkerr(S26_Sched2620_SetModePulseMeas(x, iom, chan, ActLowX));
  io_exec(x);
}


// Set counter mode for pulse generator
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//	chan: counter channel number in range 0...3
//  ActLowX: index pin polarity: 1 = active low, 0 = active high
//  HardTrig: hardware triggered by index input: 0 = disable, 1 = enable
//  t1: time of active state of output in 100ns units
//  ActLowOut: output pin polarity: 1 = active low, 0 = active high
__declspec (dllexport) void __stdcall SetModePulseGen(int mm, int iom, int chan, int ActLowX,
    int HardTrig, unsigned long t1, int ActLowOut)
{
  void *x;                // Transaction object.

  x = S26_SchedOpen(mm, 1);
  chkerr(S26_Sched2620_SetPreload(x, iom, chan, 0, 0xffffffff - t1 + 2));
  chkerr(S26_Sched2620_SetModePulseGen(x, iom, chan, ActLowX, HardTrig, ActLowOut));
  io_exec(x);
}


// Set counter mode for pwm generator
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//	chan: counter channel number in range 0...3
//  t1: time of active state of output in 100ns units
//  t2: time of inactive state of output in 100ns units
//  ActLowOut: output pin polarity: 1 = active low, 0 = active high
__declspec (dllexport) void __stdcall SetModePwmGen(int mm, int iom, int chan, unsigned long t1,
    unsigned long t2, int ActLowOut)
{
  void *x;                // Transaction object.

  x = S26_SchedOpen(mm, 1);
  chkerr(S26_Sched2620_SetPreload(x, iom, chan, 0, 0xffffffff - t1 + 2));
  chkerr(S26_Sched2620_SetPreload(x, iom, chan, 1, 0xffffffff - t2 + 2));
  chkerr(S26_Sched2620_SetModePwmGen(x, iom, chan, ActLowOut));
  io_exec(x);
}






/////////////////////////////////////////////////////////////////
// internal functions
/////////////////////////////////////////////////////////////////

// Open and init main DLL
static int dllinit(void)
{
  int err;

  // open DLL
  #if DEBUG
  // local directory
  //err = S26_DLLOpen("d:\\ntprog\\windrv\\2600\\labview\\2600demo", 1);
  err = S26_DLLOpen("c:\\2600\\labview", 1);
  #else
  // some common directory (windows\system32)
  err = S26_DLLOpen(NULL, 1);
  #endif

  if(err)
    {
    error("2600 DLL open error #%d", err);
    return(-1);
    }

  return(0);
}


// Close main DLL
static void dllclose(void)
{
  S26_DriverClose();
}


// Find a module
//  id: module ID
//  from: number of module to start from
static int findmodule(int mm, int id, int from)
{
  int err;
  int iom;
  unsigned short num;
  unsigned short ioms[16];
  unsigned char sts[16];

  if(from < 0)
    {
    err = S26_RegisterAllIoms(mm, 1000, &num, ioms, sts, 1);
    if(err)
      return(-1);
    from = 0;
    }
  // Count the connected IOMs, stop on first board found.
  for(iom = from; iom < 16; iom++)
    {
    if(ioms[iom] == id)
      {
      return(iom);
      }
    }
  return(-1);
}


// Error message
static void error(char *fmt, ... )
{
  va_list argptr;

  va_start(argptr, fmt);
  vsprintf(errstr, fmt, argptr);
  va_end(argptr);
  // print error messages to file
  #if DEBUG
    {
    FILE *f;

    if((f = fopen("c:\\2620vidll.log", "a")) == NULL)
      return;
    fprintf(f, "%s\n", errstr);
    fclose(f);
    }
  #endif

}


// Execute all scheduled i/o.
// Returns zero if successful.
static int io_exec(void *x)
{
  char status[16];		// Iom status info.
  GWERR err;

  // Execute the scheduled i/o.  Report error if one was detected.
  err = S26_SchedExecute(x, 1000, status );
  if(err)
    error("Connection error 0x%08x", err);
  return(err);
}






